---
title: Xamarin.Android warning XA1024
description: XA1024 warning code
ms.date: 07/08/2020
---
# Xamarin.Android warning XA1024

## Example messages

```
warning XA1024: Ignoring configuration file 'Foo.dll.config'. .NET configuration files are not supported in Xamarin.Android projects that target .NET 6 or higher.
```

## Issue

No support for [configuration files][config] such as `Foo.dll.config`
or `Foo.exe.config` is available in Xamarin.Android projects targeting
.NET 6 or higher. [`<dllmap>`][dllmap] configuration elements are not
supported in .NET 6 at all, and other element types for compatibility
packages like [System.Configuration.ConfigurationManager][nuget] have
never been supported in Xamarin.Android projects.

[config]: https://docs.microsoft.com/dotnet/framework/configure-apps/
[nuget]: https://www.nuget.org/packages/System.Configuration.ConfigurationManager/

## Solution

Use a supported alternative for [`<dllmap>`][dllmap], such as the
[NativeLibrary][native] API.

Use an alternative for application settings such as
[Preferences][preferences] in Xamarin.Essentials.

[dllmap]: https://github.com/dotnet/coreclr/blob/master/Documentation/design-docs/dllmap.md
[native]: https://docs.microsoft.com/dotnet/api/system.runtime.interopservices.nativelibrary
[preferences]: https://docs.microsoft.com/xamarin/essentials/preferences
